This archive contains encodings and glue code for the computation of maximin share allocations, and the generation of problem instances. The encodings are written in the ASP-Core 2 language, and intended for use with the Potassco Clingo Answer Set Programming Framework. Each encoding contains comments describing the intended effect of important statements. 

Unless otherwise noted all encodings accept as input an arbitrary connected graph, and a set of utilities for each agent. A small sample instance is included below:

```
#const n=4.
agent(1..n). resource(1..8). utils(1..4). 

edge(A,B) :- resource(A), resource(B), B - A == 1.
edge(B,A) :- B = #max{X:resource(X)}, A = #min{Y:resource(Y)}.

util(1,1,2).
util(1,2,1).
util(1,3,3).
util(1,4,2).
util(1,5,3).
util(1,6,1).
util(1,7,3).
util(1,8,1).

util(2,1,2).
util(2,2,2).
util(2,3,2).
util(2,4,2).
util(2,5,1).
util(2,6,2).
util(2,7,3).
util(2,8,2).

util(3,1,1).
util(3,2,1).
util(3,3,1).
util(3,4,2).
util(3,5,1).
util(3,6,3).
util(3,7,1).
util(3,8,2).

util(4,1,2).
util(4,2,1).
util(4,3,2).
util(4,4,2).
util(4,5,1).
util(4,6,3).
util(4,7,3).
util(4,8,1).

```

The first section defines the parameters for the number of agents, the number of resources, and the range of utilities available for assignment. This instance is a cycle so the next section simply defines the connectivity for a cycle on the resources. Finally the predicates `util(A,R,U).` describe the utility value `U`, assigned by agent `A` to resource `R`.

This input format is common to all the encodings included here. Replacing the connectivity section allows for other graphs to be generated such as a grid with constant width `x` and height `y`:

```
edge(A,B) :- resource(A), resource(B), B == A + 1, A \ x != 0.
edge(A,B) :- resource(A), resource(B), B == A + x.
```

Several example instances are included in the examples folder.

## Clingo Usage:

Clingo can be invoked from the command line as follows: `> clingo --<OPTION> = <VALUE> <FILE1>.lp <FILE2>.lp`.
 

## Using the Encodings:

The basic encodings found in MaxiMin_Encodings are `MaxMin_A.lp` and `MaxMin_B.lp`. `MaxMin_A.lp` is used to generate the guarantees for each agent, in other words the minimum utility which their bundle gives them. If any agent receives a bundle worth less than their maximin guarantee that allocation is considered unfair. `MaxMin_B.lp` takes these values for all agents and attempts to allocate all the resources fairly with respect to the MaxiMin Share criterion. 

Using Clingo from the command line we generate each guarantee by running `clingo --const t=<A> MaxMin_A.lp instance.lp` for each agent `A`. We can then add the `agentMin(A,V).` predicates found to the `instance.lp` and run `clingo MaxMin_B.lp instance.lp` to compute an allocation. 

Alternatively there are python functions available in the python folder which utilize the Clingo Python API and provide an alternative interface. 


## Using the Python Functions:

There are several useful python functions included in the Python folder. The first three functions provide generators for three different types of graphs: cycles, complete graphs, and grid graphs. The output of these functions are valid instances encoded for Clingo as in the example above. 

The next three functions provide three different methods for finding the maximin share guarantee for a particular agent, given one of the input instances generated using the functions mentioned above. One uses the default Clingo optimization statement, one uses a simple linear search, and the other uses a tuneable binary search, where the user can set a point other than the midpoint to split on during search.

Finally there are functions for solving an entire instance, for now using the default optimization statement only.

# Results

There is a clear performance increase using alternative search algorithms compared to the basic optimization statements. Trinary and quaternary simply denote different fractions used in place of the midpoint found in typical binary search. 

## Average Evaluation Time for cycles size 24-63 with different search algorithms
A ╲ B │       Base      Linear      Binary     Trinary  Quaternary
──────┼───────────────────────────────────────────────────────────
24    │   0.996622    0.818039    0.702096    0.768058    0.751706
25    │    1.07482     0.85192    0.849871    0.826119    0.830288
26    │    1.43886     1.11624     1.00184    0.972685     1.00192
27    │    1.40098     1.14172     1.08504    0.925603     0.94308
28    │      1.643     1.31223     1.16993     1.06112     1.03009
29    │    1.90115     1.49209       1.278     1.12993      1.1909
30    │    2.18002     1.79104     1.46594     1.51382     1.52486
31    │    2.92572     2.29453     1.96547     1.69915       1.701
32    │    2.79611     2.33519     1.90462     1.84601     1.83753
33    │    3.34872     2.55685     2.60691      2.0999     2.06199
34    │    3.66034     2.81064     2.59997     2.18076     2.25937
35    │    3.94214     3.16602      3.2582     2.45505     2.39689
36    │    4.58001     3.65642     3.17564     2.89349     2.99624
37    │    5.10419     4.14314     4.03522     3.43273      3.4441
38    │    5.55518       4.776     4.10558     3.88862     3.87822
39    │    6.01815     5.08529     4.42058     4.11445     4.29564
40    │    6.18297     5.42305     4.86019     4.25963      4.3345
41    │    7.64567     5.90991     5.68142     4.80425     4.77328
42    │     8.5225      6.7019     5.32009     5.21459     5.26961
43    │    8.28022     7.10924     5.77963     5.68132     5.82772
44    │     10.384     8.43875     6.87207     6.19636     6.31159
45    │      11.27     9.37585      7.3226      7.0865     7.12231
46    │    11.8937     9.62153     7.94351     7.72289     7.83933
47    │    12.3191     10.8125     8.70431     7.74581     7.78655
48    │    14.8926     11.8023     9.80764      10.098     9.96082
49    │    16.8161     13.9381     11.6797     10.2036     10.2163
50    │    17.6042     13.9875     11.8661     10.8037     10.5966
51    │     19.999     15.8755       13.16     11.0461     11.2225
52    │     22.444     16.2479     12.3412     11.7898     11.8651
53    │    22.9291     18.0168     12.5662     12.4691     12.6905
54    │    24.1723     19.0601     13.6766     13.3201     13.5856
55    │    25.1942     20.1185     15.4481     14.3357     14.1408
56    │     25.173      21.004     14.8526     14.0492      14.034
57    │    28.8252     22.0775     16.0283     15.3209     14.8473
58    │    29.7503     23.1387     16.7142     15.1858     15.2278
59    │    31.8691     24.5631     18.5746     16.7565     17.0871
60    │    38.3344     25.6273     18.7451     17.7164     18.1888
61    │    34.3384     25.7637     18.3442     17.4464     17.6386
62    │    38.1068     27.7695     18.6654     17.2824     17.3138
63    │    36.2503       27.88     20.4244     18.7667      18.768

## Maximum Evaluation Time for cycles size 24-63 with different search algorithms

A ╲ B │       Base      Linear      Binary     Trinary  Quaternary
──────┼───────────────────────────────────────────────────────────
24    │     1.9857     1.24722     1.00678     1.19289     1.30798
25    │    2.11984     1.28182     1.37598     1.13152     1.11078
26    │     2.3464     1.87499     1.57743     1.64906     1.62958
27    │    2.79365     1.75215     2.05589      1.6642     1.65384
28    │    3.21922     1.96022     2.15213     1.67004      1.5783
29    │    2.93738     2.54931     2.15787     1.81194     1.89251
30    │    4.22101     2.63019     2.85533     2.35608      2.3502
31    │    4.68739     2.81917     3.06818     2.65703     2.48185
32    │    5.32285     3.39332      3.0793     2.89828     2.86687
33    │    6.27609     3.98431     4.06625     2.95861     2.93839
34    │     6.0984     4.88528     5.45226     3.34011     4.25224
35    │     6.7995     4.67115     5.93824     3.61407     3.72966
36    │    7.67506     5.09819      5.8192     4.36409     4.33983
37    │    8.74863     5.51693     6.77401     5.33588     5.36687
38    │    8.99404     6.73054     8.14832     6.51027     6.47083
39    │       10.6     7.17484     7.35077      6.5413     6.64945
40    │     10.499     7.82557     8.87015     6.35452     6.25877
41    │    15.3022     8.27909     9.27052     6.78668     6.49545
42    │    13.0909     10.2507     9.23123     8.04733     8.61425
43    │    14.0997     10.7858     11.5929      8.1559     9.66794
44    │    18.9025     12.9011     11.2836     8.94676     8.72607
45    │    20.6267     15.6156     12.5957     9.67121     10.1884
46    │    20.4833     15.4213     18.8112     13.3083       13.42
47    │    24.2627     15.8065     13.5757      11.503     11.4105
48    │     28.421     15.8633     16.8457     14.3107     15.4531
49    │     34.527     19.8939     19.9215     14.7877     14.6534
50    │    31.0657     20.7317     18.7677     15.5439     15.9305
51    │    27.7188     23.9564     23.0886     15.7497     16.8042
52    │    51.4096     21.6663     20.8933     18.5622     18.8312
53    │    56.3782     27.8282     25.5223     17.8885     18.0895
54    │    48.0293     25.6203     27.1691     20.7228     18.7538
55    │    37.8236     26.3422     26.8929     19.8121     20.4478
56    │    35.4308     29.0574     25.2194     22.4974      22.864
57    │     76.031     34.3637     26.3165     21.0004     21.5114
58    │    55.0591      30.402     29.3121     25.8542     25.8598
59    │     56.993     36.9595     33.4075     23.0591     22.7879
60    │    132.122     34.7663     30.3987     29.2707     29.4773
61    │    57.4622      36.422     30.6401     26.7615     27.8662
62    │    108.967     39.9839      35.258     27.5573     27.7861
63    │    109.036     39.3501     31.2755     25.4884     23.9435